from path import Path
import jujuresources
from charmhelpers.core import hookenv
from charmhelpers.core import host
from charmhelpers.core import unitdata
from jujubigdata import utils
from jujubigdata.relations import HadoopPlugin, Zookeeper
from hbase_util import get_zookeeper_servers


# Extended status support
# We call update_blocked_status from the "requires" section of our service
# block, so be sure to return True. Otherwise, we'll block the "requires"
# and never move on to callbacks. The other status update methods are called
# from the "callbacks" section and therefore don't need to return True.
def update_blocked_status():
    if unitdata.kv().get('charm.active', False):
        return True
    rels = (
        ('apache-hadoop-plugin', 'Hadoop', HadoopPlugin()), 
        ('apache-zookeeper', 'Zookeeper', Zookeeper()),
    )
    missing_rel = [rel for rel, res, impl in rels if not impl.connected_units()]
    not_ready = [(rel, res) for rel, res, impl in rels if not impl.is_ready()]
    if missing_rel:
        hookenv.status_set('blocked', 'Waiting for relation to %s' % (
            ' and '.join(missing_rel),
        )),
    elif not_ready:
        unready_rels, unready_ress = zip(*not_ready)
        hookenv.status_set('waiting', 'Waiting for %s to provide %s' % (
            ' and '.join(unready_rels),
            ' and '.join(unready_ress),
        ))
    return True


def update_working_status():
    if unitdata.kv().get('charm.active', False):
        hookenv.status_set('maintenance', 'Updating configuration')
        return
    hookenv.status_set('maintenance', 'Setting up Apache hbase')


def update_active_status():
    unitdata.kv().set('charm.active', True)
    hookenv.status_set('active', 'Ready')


# Main hbase class for callbacks
class HBase(object):
    def __init__(self, dist_config):
        self.dist_config = dist_config
        self.resources = {
            'hbase': 'hbase-%s' % host.cpu_arch(),
        }
        self.verify_resources = utils.verify_resources(*self.resources.values())

    def is_installed(self):
        return unitdata.kv().get('hbase.installed')

    def install(self, force=False):
        '''
        HBase supports three installation modes:
        1 - Standalone mode (not supported) - Single node HBase instance running against the local filesystem (no HDFS)
        2 - Pseudo-Distributed (not-supported) - Single node HBase connected to HDFS and internal Zookeeper
        3 - Fully Distributed (supported)- One HBase Master, 1-n HBase RegionServer, HDFS cluster, and external Zookeeer
            quorum.
        '''
        if not force and self.is_installed():
            return
        self.dist_config.add_dirs()
        jujuresources.install(self.resources['hbase'],
                              destination=self.dist_config.path('hbase'),
                              skip_top_level=True)
        self.setup_hbase_config()
        self.install_demo()

        unitdata.kv().set('hbase.installed', True)
        unitdata.kv().flush(True)

    def install_demo(self):
        '''
        Install hbasepi.sh to /home/ubuntu (executes hbasePI example app)
        '''
        demo_source = 'scripts/hbase_test.sh'
        demo_target = '/home/ubuntu/hbase_test.sh'
        Path(demo_source).copy(demo_target)
        Path(demo_target).chmod(0o755)
        Path(demo_target).chown('ubuntu', 'hadoop')

    def setup_hbase_config(self):
        '''
        copy the default configuration files to hbase_conf property
        defined in dist.yaml
        '''
        default_conf = self.dist_config.path('hbase') / 'conf'
        hbase_conf = self.dist_config.path('hbase_conf')
        hbase_conf.rmtree_p()
        default_conf.copytree(hbase_conf)
        # Now remove the conf included in the tarball and symlink our real conf
        default_conf.rmtree_p()
        hbase_conf.symlink(default_conf)

    def configure_hbase(self):
        '''
        Configure hbase environment for all users

        hbase-env.sh:
        Script for Linux environments to set up the working environment for HBase,
        including the location of Java, Java options, and other environment variables.
        The file contains many commented-out examples to provide guidance.

        hbase-site.xml:
        which is the main HBase configuration file
        '''
        e = utils.read_etc_env()
        '''
        To toggle HBase management of ZooKeeper, use the HBASE_MANAGES_ZK variable
        in conf/hbase-env.sh. This variable, which defaults to true, tells HBase whether to
        start/stop the ZooKeeper ensemble servers as part of HBase start/stop.
        Set to false as we use external distributed apache-zookeeper
        '''
        hbase_env = self.dist_config.path('hbase_conf') / 'hbase-env.sh'
        utils.re_edit_in_place(hbase_env, {
            r'.*export JAVA_HOME*.*': 'export JAVA_HOME={}'.format(e["JAVA_HOME"]),
            r'.*export HBASE_LOG_DIR*.*': 'export HBASE_LOG_DIR={}'.format(self.dist_config.path('hbase_log')),
            r'.*export HBASE_PID_DIR*.*': 'export HBASE_PID_DIR={}'.format(self.dist_config.path('hbase_pid')),
            r'.*export HBASE_MANAGES_ZK*.*': 'export HBASE_MANAGES_ZK=false',
        })

        hbase_site = self.dist_config.path('hbase_conf') / 'hbase-site.xml'
        with utils.xmlpropmap_edit_in_place(hbase_site) as xml:
            xml['hbase.cluster.distributed'] = 'true'
            xml['hbase.rootdir'] = 'hdfs:///user/ubuntu/hbase'
            xml["hbase.zookeeper.quorum"] = get_zookeeper_servers()

        hbase_log4j = self.dist_config.path('hbase_conf') / 'log4j.properties'
        utils.re_edit_in_place(hbase_log4j, {
            r'log4j.rootCategory=INFO, console': 'log4j.rootCategory=ERROR, console',
        })

        hbase_home = self.dist_config.path('hbase')
        hbase_bin = hbase_home / 'bin'
        # update environment variables
        with utils.environment_edit_in_place('/etc/environment') as env:
            if hbase_bin not in env['PATH']:
                env['PATH'] = ':'.join([env['PATH'], hbase_bin])
            env['HBASE_CONF_DIR'] = self.dist_config.path('hbase_conf')
            env['HBASE_HOME'] = hbase_home

    def start(self):
        print("HBase start ......")
        node_type = ""
        if unitdata.kv().get('hbase_master'):
            node_type="master"
        elif unitdata.kv().get('hbase_regionserver'):
            node_type="regionserver"
        else:
            print ("not an hbase node ... will add master_backup node_type in future")
            return
        hbase_home = self.dist_config.path('hbase')
        utils.run_as('ubuntu', '{}/bin/hbase-daemon.sh'.format(hbase_home),'restart', node_type)

    def stop(self):
        print("HBase stop ......")
        node_type = ""
        if unitdata.kv().get('hbase_master'):
            node_type="master"
        elif unitdata.kv().get('hbase_regionserver'):
            node_type="regionserver"
        else:
            print ("invalide hbase node_type ... will add master_backup node type in future")
            return
        hbase_home = self.dist_config.path('hbase')
        utils.run_as('ubuntu', '{}/bin/hbase-daemon.sh'.format(hbase_home),'stop', node_type)

    def cleanup(self):
        self.dist_config.remove_dirs()
        unitdata.kv().set('hbase.installed', False)
        unitdata.kv().flush(True)
